/*
* Copyright © 2010 Red Hat, Inc.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*           http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

package com.redhat.rhevm.api.mock.resource;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import javax.ws.rs.core.Response;
import javax.ws.rs.core.UriBuilder;

import com.redhat.rhevm.api.mock.util.SimpleQueryEvaluator;
import com.redhat.rhevm.api.model.DataCenter;
import com.redhat.rhevm.api.model.Cluster;
import com.redhat.rhevm.api.model.Clusters;
import com.redhat.rhevm.api.model.CPU;
import com.redhat.rhevm.api.resource.ClusterResource;
import com.redhat.rhevm.api.resource.ClustersResource;

import static com.redhat.rhevm.api.mock.resource.AbstractMockResource.allocateId;


public class MockClustersResource extends AbstractMockQueryableResource<Cluster> implements ClustersResource {

    private static Map<String, MockClusterResource> clusters =
        Collections.synchronizedMap(new HashMap<String, MockClusterResource>());

    public MockClustersResource() {
        super(new SimpleQueryEvaluator<Cluster>());
    }

    public void populate() {
        synchronized (clusters) {
            while (clusters.size() < 2) {
                MockClusterResource resource = new MockClusterResource(allocateId(Cluster.class), getExecutor(), this);
                resource.getModel().setName("cluster" + resource.getModel().getId());
                CPU cpu = new CPU();
                cpu.setId((clusters.size() % 2) == 0 ? "Intel Xeon" : "AMD Opteron G1");
                resource.getModel().setCpu(cpu);
                DataCenter dataCenter = new DataCenter();
                dataCenter.setId(resource.getModel().getId());
                resource.getModel().setDataCenter(dataCenter);
                clusters.put(resource.getModel().getId(), resource);
            }
        }
    }

    @Override
    public Clusters list() {
        Clusters ret = new Clusters();

        for (MockClusterResource cluster : clusters.values()) {
            if (filter(cluster.getModel(), getUriInfo(), Cluster.class)) {
                ret.getClusters().add(cluster.addLinks());
            }
        }

        return ret;
    }

    @Override
    public Response add(Cluster cluster) {
        MockClusterResource resource = new MockClusterResource(allocateId(Cluster.class), getExecutor(), this);

        resource.updateModel(cluster);

        String id = resource.getId();
        clusters.put(id, resource);

        UriBuilder uriBuilder = getUriInfo().getAbsolutePathBuilder().path(id);

        cluster = resource.addLinks();

        return Response.created(uriBuilder.build()).entity(cluster).build();
    }

    @Override
    public void remove(String id) {
        clusters.remove(id);
    }

    @Override
    public ClusterResource getClusterSubResource(String id) {
        return clusters.get(id);
    }
}
