/*
* Copyright © 2010 Red Hat, Inc.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*           http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

package com.redhat.rhevm.api.mock.resource;

import java.util.concurrent.Executor;

import com.redhat.rhevm.api.model.DataCenter;
import com.redhat.rhevm.api.resource.AssignedPermissionsResource;
import com.redhat.rhevm.api.resource.AttachedStorageDomainsResource;
import com.redhat.rhevm.api.resource.DataCenterResource;
import com.redhat.rhevm.api.common.resource.UriInfoProvider;
import com.redhat.rhevm.api.common.util.JAXBHelper;
import com.redhat.rhevm.api.common.util.LinkHelper;


public class MockDataCenterResource extends AbstractMockResource<DataCenter> implements DataCenterResource {
    /* FIXME: would like to do:
     * private @Context UriInfo uriInfo;
     */

    /**
     * Package-protected ctor, never needs to be instantiated by JAX-RS framework.
     *
     * @param dataCenter  encapsulated DataCenter
     * @param executor    executor used for asynchronous actions
     */
    MockDataCenterResource(String id, Executor executor, UriInfoProvider uriProvider) {
        super(id, executor, uriProvider);
    }

    // FIXME: this needs to be atomic
    public void updateModel(DataCenter dataCenter) {
        // update writable fields only
        if (dataCenter.isSetName()) {
            getModel().setName(dataCenter.getName());
        }
        if (dataCenter.isSetDescription()) {
            getModel().setDescription(dataCenter.getDescription());
        }
        if (dataCenter.isSetStorageType()) {
            getModel().setStorageType(dataCenter.getStorageType());
        }
    }

    public DataCenter addLinks() {
        DataCenter dataCenter = JAXBHelper.clone(OBJECT_FACTORY.createDataCenter(getModel()));

        return LinkHelper.addLinks(getUriInfo(), dataCenter);
    }

    /* FIXME: kill uriInfo param, make href auto-generated? */
    @Override
    public DataCenter get() {
        return addLinks();
    }

    @Override
    public DataCenter update(DataCenter dataCenter) {
        validateUpdate(dataCenter);
        updateModel(dataCenter);
        return addLinks();
    }

    public AttachedStorageDomainsResource getAttachedStorageDomainsResource() {
        return null;
    }

    @Override
    public AssignedPermissionsResource getPermissionsResource() {
        return null;
    }
}
