/*
* Copyright © 2010 Red Hat, Inc.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*           http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

package com.redhat.rhevm.api.mock.resource;

import java.util.concurrent.Executor;


import com.redhat.rhevm.api.model.Network;
import com.redhat.rhevm.api.resource.NetworkResource;
import com.redhat.rhevm.api.common.resource.UriInfoProvider;
import com.redhat.rhevm.api.common.util.JAXBHelper;
import com.redhat.rhevm.api.common.util.LinkHelper;


public class MockNetworkResource extends AbstractMockResource<Network> implements NetworkResource {

    /**
     * Package-protected ctor, never needs to be instantiated by JAX-RS framework.
     *
     * @param network     encapsulated network
     * @param executor executor used for asynchronous actions
     */
    MockNetworkResource(String id, Executor executor, UriInfoProvider uriProvider) {
        super(id, executor, uriProvider);
    }

    // FIXME: this needs to be atomic
    public void updateModel(Network network) {
        // update writable fields only
        if (network.isSetName()) {
            getModel().setName(network.getName());
        }
        if (network.isSetDescription()) {
            getModel().setDescription(network.getDescription());
        }
    }

    public Network addLinks() {
        return LinkHelper.addLinks(getUriInfo(), JAXBHelper.clone(OBJECT_FACTORY.createNetwork(getModel())));
    }

    @Override
    public Network get() {
        return addLinks();
    }

    @Override
    public Network update(Network network) {
        validateUpdate(network);
        updateModel(network);
        return addLinks();
    }
}
